const findConfig = require('find-config');

const util = require('util');

const exec = util.promisify(require('child_process').exec);

const axios = require('axios');

const _require = require('./utils'),
      baseUrl = _require.baseUrl,
      authenticate = _require.authenticate,
      buildModuleCatalogBuilder = _require.buildModuleCatalogBuilder,
      getCombinedTranslations = _require.getCombinedTranslations,
      getTranslationsMap = _require.getTranslationsMap;

const config = findConfig.require('memsource.config.js');

const MEMSOURCE_PROJECT = config.MEMSOURCE_PROJECT,
      MEMSOURCE_FILE_IMPORT_SETTINGS_ID = config.MEMSOURCE_FILE_IMPORT_SETTINGS_ID;

const _require$setup = require('@cloudflare/cli-helpers').setup({
  cliName: 'memsource-cli'
}),
      bail = _require$setup.bail,
      success = _require$setup.success,
      warn = _require$setup.warn;

const targetLangs = ['de_de'];
const importSettings = {
  uid: MEMSOURCE_FILE_IMPORT_SETTINGS_ID
};
const memsourceSettings = {
  targetLangs,
  importSettings
};
/**
 * createMemsourceJob
 * Given a translation JSON object, send a curl request to create
 * a job in Memsource
 *
 * @param {*} token - Memsource token
 * @param {*} translation - translation JSON object
 * @param {*} remoteFileName - file name to be used in Memsource
 */

const createMemsourceJob = async (token, translation, remoteFileName) => {
  try {
    await axios.post(`${baseUrl}projects/${MEMSOURCE_PROJECT}/jobs`, translation, {
      params: {
        token
      },
      headers: {
        'content-type': 'application/octet-stream',
        'content-disposition': `filename*=UTF-8''${remoteFileName}`,
        memsource: JSON.stringify(memsourceSettings)
      }
    });
    success('Uploaded file - ' + remoteFileName);
  } catch (e) {
    bail('Failed to upload file - ' + remoteFileName + '\n' + e);
  }
};
/**
 * push
 * Determines whether to upload all translations files at once or
 * upload them one by one
 *
 * @param {string} token - Memsource token
 * @param {string} translations - translations JSON object
 * @returns {Promise} promise that resolves/rejects with succeeded/failed file name
 */


const push = async (token, translations, isCombined) => {
  if (!isCombined) {
    for (let fileName of Object.keys(translations)) {
      const remoteFileName = `${fileName.split('.')[0]}-${new Date().toISOString()}.json`;
      await createMemsourceJob(token, translations[fileName], remoteFileName);
    }
  } else {
    const remoteFileName = `combined-translations-${new Date().toISOString()}.json`;
    await createMemsourceJob(token, translations, remoteFileName);
  }
};
/**
 * up
 * This function gathers all the translations files and passes
 * them to `push(...)` for uploading to Memsource.
 *
 * @param {Object} cmd - object of command line arguments passed to memsource up
 */


const up = async cmd => {
  // Set up memsource auth
  const response = await authenticate();
  const token = response.data.token; // Write module catalogs as standard JSON catalogs

  try {
    const moduleCatalogBuilderScriptPath = await buildModuleCatalogBuilder();

    const _await$exec = await exec(`node ${moduleCatalogBuilderScriptPath}`),
          stderr = _await$exec.stderr,
          stdout = _await$exec.stdout;

    if (stdout) {
      success(stdout);
    }

    if (stderr) {
      throw new Error(stderr);
    }
  } catch (e) {
    // Since module catalogs are experimental, let's just warn and continue
    warn('Error creating module catalogs');
    warn(e);
  } // Combine individual json files into one combined.json


  const translations = getTranslationsMap();
  const combinedTranslations = getCombinedTranslations(translations); // Push the combined translations to memsource

  push(token, combinedTranslations, cmd.combined === true);
};

module.exports = up;