function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? Object(arguments[i]) : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const fs = require('fs');

const axios = require('axios');

const findConfig = require('find-config');

const path = require('path');

const glob = require('glob');

const util = require('util');

const writeJsonFile = require('write-json-file');

const fromPairs = require('lodash.frompairs');

const writeFileAsync = util.promisify(fs.writeFile);
const mkdirp = util.promisify(require('mkdirp'));

const _require$setup = require('@cloudflare/cli-helpers').setup({
  cliName: 'memsource-cli'
}),
      bail = _require$setup.bail;

const config = findConfig.require('memsource.config.js');

const MEMSOURCE_USER = config.MEMSOURCE_USER,
      MEMSOURCE_PASS = config.MEMSOURCE_PASS;
const readdirSync = fs.readdirSync,
      statSync = fs.statSync;
/*
 * Constants
 */

const baseUrl = 'https://cloud.memsource.com/web/api2/v1/';
const dashPkgName = '@cloudflare/app-dash';
const CATALOGS_ROOT = '@cloudflare/intl-translations';
const validStatus = 'DELIVERED';
const localeMap = {
  de_de: 'de-DE'
};
const outputDir = 'en-US/generated';
/*
 * Selectors
 */

const getTranslationPackageRoot = () => path.dirname(require.resolve(`${CATALOGS_ROOT}/package.json`));

const getLocaleRoot = () => `${getTranslationPackageRoot()}/src/locale/`;

const getDashRoot = () => path.dirname(require.resolve(`${dashPkgName}/package.json`));

const getTranslationFilePaths = (locale = 'en-US') => glob.sync(`${getLocaleRoot()}${locale}/*.json`).concat(glob.sync(path.join(getModuleCatalogsOutputPath(), '*.json')));

const getModuleCatalogsFilePaths = () => glob.sync(`${getDashRoot()}/**/*.translations.ts`);

const getModuleCatalogNameFromPath = filePath => path.basename(filePath).replace('.translations.ts', '');

const getModuleCatalogsOutputPath = () => path.join(getTranslationPackageRoot(), `/src/locale/${outputDir}`);
/**
 * getTranslationsMap
 * Maps the json translation files and return a Map that looks like
 * This function requires the files from the input paths and builds a map of
 * filesNames -> translations
 *
 *  Map {
 *    'access.json' => { hello: 'hello' },
 *    'worker.json' => { world: 'world' }
 *  }
 *
 * @return {Map} map - {<filename>:<translatoions>}
 */


const getTranslationsMap = (locale = 'en-US') => {
  const translations = new Map();
  getTranslationFilePaths(locale).forEach(filePath => {
    const translationFileName = path.basename(filePath),
          translationPhrases = require(filePath);

    translations.set(translationFileName, translationPhrases);
  });
  return translations;
};
/**
 * getCombinedTranslations
 *
 * Gets the final combined JSON object. Also does a check for duplicate translations
 * before building the object.
 *
 * @param {Map} map - {<key>: {<translations>}}
 * @return {Object} object- {<filename>:<translations>}
 */


const getCombinedTranslations = translationsMap => {
  const duplicateTranslations = getDuplicateTranslations(translationsMap); // If we find a duplicate string, we cant recover so throw an error

  if (duplicateTranslations.length !== 0) {
    bail(`Found duplicate translation key ${JSON.stringify(duplicateTranslations)}. Canceling upload and exiting.`);
  } // Convert the translations map to an object


  const translationsCombined = fromPairs([...translationsMap]);
  return translationsCombined;
};
/**
 * getDuplicateTranslations
 * Gets duplicate transaltion keys in translation json files if any.
 *
 * This function returns an array of duplicte keys.
 * This array could be empty if no duplicates are found.
 *
 * Note: This function can only catch duplicates across different files.
 * It cannot detect duplicates in the same file.
 * It expects an input of a Map with the following structure
 *  Map {
 *    'access.json' => { hello: 'hello' },
 *    'worker.json' => { world: 'world' }
 *  }
 * @param {Map} map - {<key>: {<translations>}}
 * @return {Array} array- {[<duplicate keys>]}
 */


const getDuplicateTranslations = translationsMap => {
  const tmp = new Set(),
        duplicates = [];
  translationsMap.forEach(translations => {
    for (var key in translations) {
      if (tmp.has(key)) {
        duplicates.push(key);
      } else {
        tmp.add(key);
      }
    }
  });
  return duplicates;
};
/**
 * getModuleCatalogBuildScript
 * Gets the build script that we will write, run through webpack, and
 * then execute to generate regular json catalogs from module catalogs
 * @param {string[]} catalogPaths - Array of paths to to module catalogs
 * @return {string} The build script
 */


const getModuleCatalogBuildScript = (catalogPaths = getModuleCatalogsFilePaths()) => `
  require('babel-polyfill')

  const fs = require('fs')
  const util = require('util')
  const path = require('path')
  const mkdirp = util.promisify(require('mkdirp'))
  const { bail, success, warn } = require('@cloudflare/cli-helpers').setup({
    cliName: 'memsource-cli'
  });;
  const writeFileAsync = util.promisify(fs.writeFile)

  const catalogs = {
    ${catalogPaths.map(p => [getModuleCatalogNameFromPath(p), p]).map(([name, pathname]) => `"${name}": require("${pathname}")`).join(',\n')}
  }

  Promise.all(Object.keys(catalogs).map(catalogName => {
    const catalog = catalogs[catalogName]

    return mkdirp("${getModuleCatalogsOutputPath()}").then(() => writeFileAsync(
      path.join("${getModuleCatalogsOutputPath()}", catalogName) + '.json',
      JSON.stringify(catalog.translations, null, '  ')
    )).then(() => success('Module Catalog Generated: ' + catalogName)).catch(e => {
      warn('Error writing: ' + catalogName)
      warn(e)
    })
  }))
  .then(() => process.exit(0))
  .catch(e => bail(e))
`;
/**
 * buildModuleCatalogBuilder
 * Uses getModuleCatalogBuildScript to build a script which goes through
 * all `.translations.ts` files and produces an equivalent `catalog.json`
 * file, whose name corresponds to the prefix of the Module Catalog.
 * Since these Module Catalogs need to be run through webapck, we
 * generate a file which is run through webpack, and the resulting filename
 * is returned
 * @return {string} The output path of the build script
 */


const buildModuleCatalogBuilder = async () => {
  const entry = path.join(__dirname, '../lib/build-module-catalogs.js');
  const filename = 'output-module-catalogs-json.js';
  const output = {
    filename,
    path: path.join(__dirname, '../lib')
  };
  await mkdirp(path.join(__dirname, '../lib'));
  await writeFileAsync(entry, getModuleCatalogBuildScript());
  await new Promise((resolve, reject) => require('webpack')(_objectSpread({}, require('./webpack.config'), {
    entry,
    output
  }), error => {
    if (error) return reject(error);
    resolve();
  }));
  return path.join(__dirname, '../lib', filename);
}; // API utils


const authenticate = () => {
  return axios.post(`${baseUrl}auth/login`, {
    userName: MEMSOURCE_USER,
    password: MEMSOURCE_PASS
  });
};
/**
 * writeFormattedJsonFile
 * Writes JSON file with indent:2 and detectIndent:true
 *
 * @param {string} filePath - file path
 * @param {content} - Contents to write
 * @return {Promise} promise- {Resolves if write success, else fails}
 */


const writeFormattedJsonFile = (filePath, content) => {
  return writeJsonFile.sync(filePath, content, {
    indent: 2
  });
};
/**
 * getSplitTranslations
 * Splits combined translations into an array of arrays
 * For example
 *  {
 *    'access.json' : { hello: 'hello' },
 *    'worker.json' : { world: 'world' }
 *  }
 *  Becomes
 * [ [ 'access.json',{ hello: 'hello' } ],
 *   [ 'workers.json', { world: 'world' } ] ];
 *
 * @param {object} phrases - Object {<key>:{<translations>}}
 * @param {content} - Contents to write
 * @return {Promise} promise- {Resolves if write success, else fails}
 */


const getSplitTranslations = phrases => Object.entries(phrases);
/**
 * getNumOfLinks
 * Match the regex on the given markdown to count the
 * number of hyperlinks in it.
 *
 * @param {String} markdown
 * @return {Number} # of hyperlinks in given markdown
 */


const getNumOfLinks = markdown => {
  // match links in the format: [text here](link here)
  const checkLink = /\[(.*?)\]\((.*?)\)/g;
  return (markdown.match(checkLink) || []).length;
};
/**
 * getLocales
 * Gets a list of the available locales in the locale/ directory.
 *
 * @return {String[]} locales in the locale/ directory
 */


const getLocales = () => {
  const directories = readdirSync(getLocaleRoot()).filter(f => statSync(path.join(getLocaleRoot(), f)).isDirectory());
  const outputDirIndex = directories.indexOf(outputDir); // we don't want to include the output directory

  if (outputDirIndex > -1) {
    directories.splice(outputDirIndex, 1);
  }

  return directories;
};

module.exports = {
  authenticate,
  baseUrl,
  buildModuleCatalogBuilder,
  getCombinedTranslations,
  getDuplicateTranslations,
  getModuleCatalogBuildScript,
  getModuleCatalogNameFromPath,
  getModuleCatalogsFilePaths,
  getSplitTranslations,
  getTranslationFilePaths,
  getTranslationPackageRoot,
  getTranslationsMap,
  localeMap,
  validStatus,
  writeFormattedJsonFile,
  getNumOfLinks,
  getLocales
};